#if !defined(RT_SURFACE_H)
#define RT_SURFACE_H
//------------------------------------------------------------
// surface.h
//
// Helper functions for manipulating surfaces.
//
#include <atlbase.h>    // ATLASSERT(), CComPtr<>
#include <d3d8.h>       // IDirect3DSurface8

namespace rt {

//------------------------------------------------------------
// surface_lock
//
// Helper class that locks a surface in its c'tor and unlocks
// it in its d'tor.  Provides accessors to the locked region.
//
class surface_lock
{
private:
    // Pitch is specified in bytes, not pixels.
    const BYTE *scanline(UINT y) const
    {
        return static_cast<const BYTE *>(m_data.pBits)
            + m_data.Pitch*y;
    }
    BYTE *scanline(UINT y)
    {
        return static_cast<BYTE *>(m_data.pBits) +
            m_data.Pitch*y;
    }

    CComPtr<IDirect3DSurface8> m_surface;
    D3DLOCKED_RECT m_data;

public:
    surface_lock(IDirect3DSurface8 *surface,
        DWORD flags = 0,
        const RECT *locked_region = NULL)
        : m_surface(surface)
    {
        THR(m_surface->LockRect(&m_data,
            locked_region, flags));
    }
    ~surface_lock()
    {
        // destructors should never throw exceptions, so
        // we don't use THR() here.  Also, we will never
        // be here unless the LockRect succeeded and
        // constructed a surface_lock, so UnlockRect
        // should always succeed.  We check anyway by
        // asserting success on the returned HRESULT.
        //
        // ATLASSERT is compiled out on optimized builds,
        // so use two statements because this:
        //      ATLASSERT(SUCCEEDED(m_surface->Unlock()))
        // would compile away the Unlock and introduce
        // a bug on a release build.
        const HRESULT hr = m_surface->UnlockRect();
        ATLASSERT(SUCCEEDED(hr));
    }

    // 8 bits per pixel: 1 pixel = 1 BYTE
    const BYTE *scanline8(UINT y) const
    {
        return scanline(y);
    }
    BYTE *scanline8(UINT y)
    {
        return scanline(y);
    }

    // 16 bits per pixel: 1 pixel = 1 WORD
    const WORD *scanline16(UINT y) const
    {
        return reinterpret_cast<const WORD *>(scanline(y));
    }
    WORD *scanline16(UINT y)
    {
        return reinterpret_cast<WORD *>(scanline(y));
    }

    // 24 bits per pixel: 1 pixel = 3 BYTEs
    const BYTE *scanline24(UINT y) const
    {
        return scanline(y);
    }
    BYTE *scanline24(UINT y)
    {
        return scanline(y);
    }

    // 32 bits per pixel: 1 pixel = 1 DWORD
    const DWORD *scanline32(UINT y) const
    {
        return reinterpret_cast<const DWORD *>(scanline(y));
    }
    DWORD *scanline32(UINT y)
    {
        return reinterpret_cast<DWORD *>(scanline(y));
    }
};  // surface_lock

};  // rt

#endif
